﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TXTextControl.Proofing;

namespace txspell_grid
{
    public partial class frmSpelling : Form
    {
        // private fields and member variables
        private DataGridView dataGridView;
        private DataGridViewCell curCell = null;
        private TXSpellChecker txSpellChecker1;

        public frmSpelling(DataGridView DataGridView, TXSpellChecker TXSpellChecker)
        {
            InitializeComponent();

            txSpellChecker1 = TXSpellChecker;
            txSpellChecker1.Create();
            dataGridView = DataGridView;
            
            //get the first cell text
            GetNextCellText();

            // check whether a user dictionary is available
            foreach (Dictionary dic in txSpellChecker1.Dictionaries)
            {
                if (dic is UserDictionary == false)
                    continue;

                if (((UserDictionary)dic).IsEditable)
                {
                    // enable the button in the dialog
                    btnAddToDictionary.Enabled = true;
                    break;
                }
            }
        }

        // *** GetNextCell ***
        // Iterates through the whole DataGridView and returns the
        // next DataGridViewCell.
        // From left to right, top to bottom
        private DataGridViewCell GetNextCell(DataGridViewCell CurrentDataGridViewCell)
        {
            if (CurrentDataGridViewCell == null)
                return dataGridView[0, 0];

            int newColIndex = CurrentDataGridViewCell.ColumnIndex;
            int newRowIndex = CurrentDataGridViewCell.RowIndex;

            if(CurrentDataGridViewCell.ColumnIndex != dataGridView.ColumnCount - 1)
                newColIndex++;
            else
            {
                newColIndex = 0;

                if (newRowIndex != dataGridView.RowCount - 1)
                    newRowIndex++;
                else
                    return null;
            }

            return dataGridView[newColIndex, newRowIndex];
        }

        // *** GetNextCellText ***
        // Gets the text of the next cell and checks, if spell checking
        // is complete
        private void GetNextCellText()
        {
            curCell = GetNextCell(curCell);
            curCell.Selected = true;

            string sCellText = String.Empty;

            if (curCell.Value == null || curCell.Value is DBNull)
            {
                MessageBox.Show("Spell checking complete.");
                this.Hide();
            }
            else
            {
                sCellText = (string)curCell.Value;
                rtbTextDisplay.Text = sCellText;
                txSpellChecker1.Check(sCellText);
                GetNextWord();
            }
        }

        // *** GetNextWord ***
        // This method is called recursively until the IncorrectWords
        // collection is empty (cell text is checked completely)
        private void GetNextWord()
        {
            // get new cell text
            if (txSpellChecker1.IncorrectWords.Count == 0)
                GetNextCellText();
            else
            {
                // create new suggestions for the current misspelled word
                txSpellChecker1.CreateSuggestions(txSpellChecker1.IncorrectWords[0].Text, 10);
                
                // highlight the misspelled word in the RichTextBox
                rtbTextDisplay.SelectAll();
                rtbTextDisplay.SelectionColor = Color.Black;
                rtbTextDisplay.Select(txSpellChecker1.IncorrectWords[0].Start,
                    txSpellChecker1.IncorrectWords[0].Length);
                rtbTextDisplay.SelectionColor = Color.Red;
                rtbTextDisplay.DeselectAll();

                // clear the suggetions list box
                lbSuggestions.Items.Clear();

                // add the new suggestons
                foreach (Suggestion suggestion in txSpellChecker1.Suggestions)
                {
                    lbSuggestions.Items.Add(suggestion.Text);
                }

                if (lbSuggestions.Items.Count > 0)
                {
                    lbSuggestions.SelectedIndex = 0;
                    btnChange.Enabled = true;
                }
                else
                    btnChange.Enabled = false;
            }
        }

        private void btnIgnoreOnce_Click(object sender, EventArgs e)
        {
            // ignore the word and remove it from the IncorrectWords
            txSpellChecker1.IncorrectWords.RemoveAt(0);
            GetNextWord();
        }

        private void btnChange_Click(object sender, EventArgs e)
        {
            // replace the text in the current cell
            curCell.Value = ((string)curCell.Value).Replace(
                txSpellChecker1.IncorrectWords[0].Text,
                lbSuggestions.SelectedItem.ToString());

            txSpellChecker1.IncorrectWords.RemoveAt(0);
            GetNextWord();
        }

        private void lbSuggestions_SelectedIndexChanged(object sender, EventArgs e)
        {
            btnChange.Enabled = true;
        }

        private void btnOptions_Click(object sender, EventArgs e)
        {
            txSpellChecker1.OptionsDialog();
        }

        private void btnAddToDictionary_Click(object sender, EventArgs e)
        {
            // add the misspelled word to ALL user dictionaries
            foreach (Dictionary dic in txSpellChecker1.Dictionaries)
            {
                if (dic is UserDictionary == false)
                    continue;

                if (((UserDictionary)dic).IsEditable)
                {
                    ((UserDictionary)dic).AddWord(txSpellChecker1.IncorrectWords[0].Text);
                }
            }

            txSpellChecker1.IncorrectWords.RemoveAt(0);
            GetNextWord();
        }
    }
}
